      program roe
      implicit real*8 (a-h,o-z)
      parameter(nt=80)
      parameter(nx=100)
      parameter(itorder=2)
      parameter(gamma=1.40d0)
      dimension q(3,0:nx),ql(3,nx),qr(3,nx)
      dimension flx(3,nx),flxco(3,nx)
      dimension qt(3,0:nx)
      common /flag/imuscl, imusclsuperbee, ico, icosuperbee

C initial condition etc.
      call initial(nx,gamma,rho0,rho1,p0,p1,u0,u1,q)
      t=0.0d0
      tend=14.40d0
      dt=tend/dble(nt)
      dx=1.0d0
      dtdx=dt/dx
      imuscl=0
      imusclsuperbee=0
      ico=3
      icosuperbee=1

      if (itorder.eq.1) then
C 1st order in time
        do it=1,nt
          call copyvariable(nx,q,qt)
          call makeleftright(nx,gamma,q,qt,ql,qr)
          call roescheme(nx,gamma,ql,qr,flx,flxco)
          if (ico.ne.0) then
            call addcoflux(nx,flxco,flx)
          endif
          call advance(nx,dtdx,flx,qt,q)
          t=t+dt
        enddo
      endif

      if (itorder.eq.2) then
C 2nd order in time
        do it=1,nt
C first step
          imuscl2=imuscl
          imuscl=0
          call copyvariable(nx,q,qt)
          call makeleftright(nx,gamma,q,qt,ql,qr)
          call roescheme(nx,gamma,ql,qr,flx,flxco)
          call advance(nx,dtdx/2.0d0,flx,q,qt)
          imuscl=imuscl2
C second step
          ico2=ico
          ico=0
          call makeleftright(nx,gamma,q,qt,ql,qr)
          call roescheme(nx,gamma,ql,qr,flx,flxco)
          if (ico2.ne.0) then
            call addcoflux(nx,flxco,flx)
          endif
          call copyvariable(nx,q,qt)
          call advance(nx,dtdx,flx,qt,q)
          ico=ico2
          t=t+dt
        enddo
      endif
C output etc.
      open(10,file='out.d',status='unknown',form='formatted')
      do i=1,nx
        write(10,*) dble(i),
     $    q(1,i),q(2,i)/q(1,i),
     $    (gamma-1.0d0)*(q(3,i)-q(2,i)**2/q(1,i)/2.0d0)
      enddo
      close(10)
      stop
      end

      subroutine roescheme(nx,gamma,ql,qr,flx,flxco)
      implicit real*8 (a-h,o-z)
      dimension ql(3,nx),qr(3,nx),flx(3,nx),flxco(3,nx)
      dimension rm(3),sr(3,3),sl(3,3)
      dimension fl(3),fr(3),fa(3)
      common /flag/imuscl, imusclsuperbee, ico, icosuperbee

      gam1=gamma-1.0d0
      do i=1,nx
        call variables(nx,i,ql,gamma,rl,ul,el,pl,hl)
        call variables(nx,i,qr,gamma,rr,ur,er,pr,hr)
        call roeaverage(rl,ul,hl,rr,ur,hr,au,ah)
C left(sl), right(sr) eigen vectors, eigen values(rm)
        call makematrix(gam1,au,ah,sl,sr,rm)
        call fluxatmesh(rl,ul,pl,el,fl)
        call fluxatmesh(rr,ur,pr,er,fr)
        call soundspeed(gamma,pl,rl,cl)
        call soundspeed(gamma,pr,rr,cr)
        call fluxabsA(rm,sl,sr,rl,ul,el,cl,rr,ur,er,cr,fa)
        do j=1,3
          flx(j,i)=(fl(j)+fr(j)-fa(j))/2.0d0
        enddo
        if (ico.ne.0) then
          call coscheme(nx,i,ql,qr,sl,sr,rm,flxco)
        endif
      enddo
      return
      end

      subroutine advance(nx,dtdx,flx,qp,qn)
      implicit real*8 (a-h,o-z)
      dimension qp(3,0:nx),qn(3,0:nx),flx(3,nx)
C flx(i)=FLUX(i-1/2)
      do i=1,nx-1
        do j=1,3
          qn(j,i)=qp(j,i)-dtdx*(flx(j,i+1)-flx(j,i))
        enddo
      enddo
      return
      end

      subroutine makeleftright(nx,gamma,q,qt,ql,qr)
      implicit real*8 (a-h,o-z)
      dimension q(3,0:nx),qt(3,0:nx),ql(3,nx),qr(3,nx)
      common /flag/imuscl, imusclsuperbee, ico, icosuperbee

C ql(i)=QL(i-1/2), qr(i)=QR(i-1/2)

      if (imuscl.eq.0) then
C no-MUSCL
        do i=1,nx
          do j=1,3
            ql(j,i)=qt(j,i-1)
            qr(j,i)=qt(j,i)
          enddo
        enddo
        return
      endif
C MUSCL
      call muscl(nx,gamma,q,qt,ql,qr)
      return
      end

      subroutine makematrix(gam1,u,h,sl,sr,rm)
      implicit real*8 (a-h,o-z)
      dimension rm(3),sr(3,3),sl(3,3)
C temporary
      c=dsqrt((h-u**2/2.0d0)*gam1)
      eta=gam1/c**2
C eigen values
      rm(1)=u-c
      rm(2)=u
      rm(3)=u+c
C
C right eigen vectors
C sr1
      sr(1,1)=1.0d0
      sr(2,1)=(u-c)
      sr(3,1)=(h-c*u)
C sr2
      sr(1,2)=1.0d0
      sr(2,2)=u
      sr(3,2)=u**2/2.0d0
C sr3
      sr(1,3)=1.0d0
      sr(2,3)=(u+c)
      sr(3,3)=(h+c*u)
C
C left eigen vectors
C sl1
      sl(1,1)=(eta/2.0d0*u**2+u/c)/2.0d0
      sl(1,2)=-(eta*u+1.0d0/c)/2.0d0
      sl(1,3)=eta/2.0d0
C sl2
      sl(2,1)=-eta*u**2/2.0d0+1.0d0
      sl(2,2)=eta*u
      sl(2,3)=-eta
C sl3
      sl(3,1)=(eta/2.0d0*u**2-u/c)/2.0d0
      sl(3,2)=-(eta*u-1.0d0/c)/2.0d0
      sl(3,3)=eta/2.0d0
C
      return
      end

      subroutine initial(nx,gamma,rho0,rho1,p0,p1,u0,u1,q)
      implicit real*8 (a-h,o-z)
      dimension q(3,0:nx)

C standard shock tube : u0=0.0d0
      u0=0.0d0
C sonic point check : u0=0.60d0
c      u0=0.60d0
      u1=u0

      rratio=0.1250d0
      pratio=0.10d0

      rho0=1.0d0
      rho1=rho0*rratio
      p0=1.0d0
      p1=p0*pratio

      do i=0,nx
        if (i.lt.nx/2) then
          q(1,i)=rho0
          q(2,i)=rho0*u0
          q(3,i)=rho0*u0**2/2.0d0+p0/(gamma-1.0d0)
        else
          q(1,i)=rho1
          q(2,i)=rho1*u1
          q(3,i)=rho1*u1**2/2.0d0+p1/(gamma-1.0d0)
        endif
      enddo
      return
      end

      subroutine variables(nn,i,qq,gamma,rho,u,e,p,h)
      implicit real*8 (a-h,o-z)
      dimension qq(3,nn)
      rho=qq(1,i)
      u=qq(2,i)/rho
      e=qq(3,i)
      p=(e-rho*u**2/2.0d0)*(gamma-1.0d0)
      h=(e+p)/rho
      return
      end

      subroutine soundspeed(gamma,p,rho,c)
      implicit real*8 (a-h,o-z)
      c=dsqrt(gamma*p/rho)
      return
      end

      subroutine addcoflux(nx,flxco,flx)
      implicit real*8 (a-h,o-z)
      dimension flxco(3,nx),flx(3,nx)
      do i=1,nx
        do j=1,3
          flx(j,i)=flx(j,i)+flxco(j,i)
        enddo
      enddo
      return
      end

      subroutine copyvariable(nx,q,qt)
      implicit real*8 (a-h,o-z)
      dimension q(3,0:nx),qt(3,0:nx)
      do i=0,nx
        do j=1,3
          qt(j,i)=q(j,i)
        enddo
      enddo
      return
      end

